from django.utils.decorators import make_middleware_decorator

from .middleware import Log4DjangoAuthenticationMiddleware


authenticate = make_middleware_decorator(Log4DjangoAuthenticationMiddleware)


import logging

from django.core.cache import cache

from tastypie.authentication import Authentication
from tastypie.http import HttpUnauthorized

from ..models import App


log = logging.getLogger(__name__)


class AppAuthentication(Authentication):

    HEADER_NAME = 'HTTP_AUTHORIZATION'

    def is_authenticated(self, request, **kwargs):
        app_key = self.get_identifier(request)
        try:
            log.debug('Api authentication parsed credentials: %s', app_key)
            app = self.get_app(request)
        except (App.DoesNotExist, App.MultipleObjectsReturned):
            log.exception('Api authentication not successful.')
            return HttpUnauthorized()
        return isinstance(app, App)

    def get_identifier(self, request):
        app_key = request.GET.get('app_key') or request.POST.get('app_key')
        try:
            if not app_key and self.HEADER_NAME in request.META:
                log.debug('Api authentication header received: %s', request.META[self.HEADER_NAME])
                app_key = request.META[self.HEADER_NAME].split(':', 1)[1].strip()
        except IndexError:
            log.exception('Invalid Authorization header detected: %s', request.META[self.HEADER_NAME])
            return HttpUnauthorized()
        return app_key

    def get_app(self, request):
        app_key = self.get_identifier(request)
        cache_key = 'log4django__app_{0}'.format(app_key)
        app = cache.get(cache_key, None)
        if app is None:
            app = App.objects.get(key=app_key)
            cache.set(cache_key, app)
        return app


from django.http import HttpResponse
from django.views.decorators.http import require_GET

from . import _filter_records
from ..exports.csv import Export as CsvExport
from ..decorators import authenticate
from ..settings import CSV_DOWNLOAD_FILE_NAME


@authenticate()
@require_GET
def csv(request):
    logrecords_qs = _filter_records(request)
    export = CsvExport(logrecords_qs)
    response = HttpResponse(export.render(), content_type='application/csv')
    response['Content-Disposition'] = 'attachment; filename="{0}"'.format(CSV_DOWNLOAD_FILE_NAME)
    return response


from django.db.models import Q

from ..models import LogRecord


def _filter_records(request):
    getvars = request.GET
    logrecord_qs = LogRecord.objects.all().select_related('app')
    # Filtering by get params.
    if getvars.get('q'):
        q = getvars.get('q')
        logrecord_qs = logrecord_qs.filter(
            Q(app__name__icontains=q)
            | Q(message__icontains=q)
            | Q(fileName__icontains=q)
            | Q(loggerName__icontains=q)
            | Q(exception_message__icontains=q)
            | Q(request_id__icontains=q)
            | Q(_extra__icontains=q)
        )
    if getvars.get('app'):
        logrecord_qs = logrecord_qs.filter(app_id=getvars.get('app'))
    if getvars.get('logger'):
        logrecord_qs = logrecord_qs.filter(loggerName=getvars.get('logger'))
    if getvars.getlist('level'):
        logrecord_qs = logrecord_qs.filter(level__in=getvars.getlist('level'))
    if getvars.get('from'):
        logrecord_qs = logrecord_qs.filter(timestamp__gte=getvars.get('from'))
    if getvars.get('to'):
        logrecord_qs = logrecord_qs.filter(timestamp__lte=getvars.get('to'))
    return logrecord_qs


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'LogRecord'
        db.create_table(u'log4django_logrecord', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('app', self.gf('django.db.models.fields.related.ForeignKey')(related_name='records', to=orm['log4django.App'])),
            ('loggerName', self.gf('django.db.models.fields.CharField')(max_length=225)),
            ('level', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('timestamp', self.gf('django.db.models.fields.DateTimeField')()),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('fileName', self.gf('django.db.models.fields.TextField')()),
            ('lineNumber', self.gf('django.db.models.fields.PositiveIntegerField')(null=True)),
            ('thread', self.gf('django.db.models.fields.CharField')(max_length=225, null=True, blank=True)),
            ('exception_message', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('exception_traceback', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('_extra', self.gf('django.db.models.fields.TextField')(default='{}')),
        ))
        db.send_create_signal(u'log4django', ['LogRecord'])

        # Adding model 'App'
        db.create_table(u'log4django_app', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('key', self.gf('django.db.models.fields.CharField')(default='9100f401-106f-433a-9f56-9a596dc9008e', unique=True, max_length=36)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'log4django', ['App'])


    def backwards(self, orm):
        # Deleting model 'LogRecord'
        db.delete_table(u'log4django_logrecord')

        # Deleting model 'App'
        db.delete_table(u'log4django_app')


    models = {
        u'log4django.app': {
            'Meta': {'ordering': "('name',)", 'object_name': 'App'},
            'description': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'default': "'6607271c-a797-4891-a5f9-9d9db8a71ac9'", 'unique': 'True', 'max_length': '36'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'log4django.logrecord': {
            'Meta': {'ordering': "('-id',)", 'object_name': 'LogRecord'},
            '_extra': ('django.db.models.fields.TextField', [], {'default': "'{}'"}),
            'app': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'records'", 'to': u"orm['log4django.App']"}),
            'exception_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'exception_traceback': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fileName': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'lineNumber': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'loggerName': ('django.db.models.fields.CharField', [], {'max_length': '225'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'thread': ('django.db.models.fields.CharField', [], {'max_length': '225', 'null': 'True', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {})
        }
    }

    complete_apps = ['log4django']


import uuid
import logging
import json

from django.db import models
from django.utils.translation import ugettext as _

from model_utils import Choices


class LogRecord(models.Model):
    LEVEL = Choices(
        (logging.NOTSET, 'NOTSET', _('NOTSET')),
        (logging.DEBUG, 'DEBUG', _('DEBUG')),
        (logging.INFO, 'INFO', _('INFO')),
        (logging.WARNING, 'WARNING', _('WARNING')),
        (logging.ERROR, 'ERROR', _('ERROR')),
        (logging.CRITICAL, 'CRITICAL', _('CRITICAL'))
    )

    app = models.ForeignKey('App', related_name='records')
    loggerName = models.CharField(max_length=225)
    level = models.PositiveSmallIntegerField(choices=LEVEL, default=LEVEL.NOTSET)
    timestamp = models.DateTimeField()
    message = models.TextField()
    fileName = models.TextField()
    lineNumber = models.PositiveIntegerField(null=True)
    thread = models.CharField(max_length=225, blank=True, null=True)
    exception_message = models.TextField(blank=True, null=True)
    exception_traceback = models.TextField(blank=True, null=True)
    request_id = models.CharField(max_length=36, db_index=True, blank=True, null=True)
    _extra = models.TextField(default='{}')

    @property
    def has_extra(self):
        return self._extra != LogRecord._meta.get_field('_extra').default

    @property
    def extra(self):
        return json.loads(self._extra)

    @extra.setter
    def extra(self, data):
        self._extra = json.dumps(data)

    def __unicode__(self):
        return self.message

    class Meta:
        ordering = ('-id',)


class App(models.Model):
    key = models.CharField(unique=True, max_length=36, default=lambda: str(uuid.uuid4()))
    name = models.CharField(max_length=100)
    description = models.TextField()

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('name',)



import logging

from .formatters import ModelFormatter, GearmanFormatter
from .settings import GERMAN_TASK_NAME


class ModelHandler(logging.Handler):

    def __init__(self, level=logging.NOTSET):
        super(ModelHandler, self).__init__(level)
        self.formatter = ModelFormatter()

    def emit(self, record):
        try:
            self.format(record).save()
        except Exception:
            self.handleError(record)


class GearmanHandler(logging.Handler):

    def __init__(self, level=logging.NOTSET):
        super(GearmanHandler, self).__init__(level)
        self.formatter = GearmanFormatter()

    def emit(self, record):
        from django_gearman_commands import submit_job
        try:
            submit_job(GERMAN_TASK_NAME, self.format(record))
        except Exception:
            self.handleError(record)


import json

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe
from django.utils.text import normalize_newlines

from ..models import LogRecord
from ..settings import EXTRA_DATA_INDENT, PAGINATOR_RANGE


register = template.Library()


@register.simple_tag
def level_css_class(level):
    return {
        LogRecord.LEVEL.NOTSET: 'label-default',
        LogRecord.LEVEL.DEBUG: 'label-success',
        LogRecord.LEVEL.INFO: 'label-info',
        LogRecord.LEVEL.WARNING: 'label-warning',
        LogRecord.LEVEL.ERROR: 'label-primary',
        LogRecord.LEVEL.CRITICAL: 'label-danger'
    }[int(level)]


@register.simple_tag
def extra_data(record):
    return json.dumps(record.extra, indent=EXTRA_DATA_INDENT)


@register.inclusion_tag('log4django/bootstrap/templatetags/pagination.html', takes_context=True)
def pagination(context, page):
    if PAGINATOR_RANGE > page.paginator.num_pages:
        range_length = page.paginator.num_pages
    else:
        range_length = PAGINATOR_RANGE
    range_length -= 1
    range_min = max(page.number - (range_length / 2), 1)
    range_max = min(page.number + (range_length / 2), page.paginator.num_pages)
    range_diff = range_max - range_min
    if range_diff < range_length:
        shift = range_length - range_diff
        if range_min - shift > 0:
            range_min -= shift
        else:
            range_max += shift
    page_range = range(range_min, range_max + 1)
    getvars = context['request'].GET.copy()
    getvars.pop('page', None)
    return dict(
        page=page, page_range=page_range, getvars=getvars
    )

@register.filter
@stringfilter
def remove_newlines(text):
    normalized_text = normalize_newlines(text)
    return mark_safe(normalized_text.replace('\n', ' '))
remove_newlines.is_safe = True



from django.utils.importlib import import_module

from tastypie.authorization import Authorization
from tastypie.exceptions import ImmediateHttpResponse
from tastypie.resources import ModelResource
from tastypie import fields

from ..models import LogRecord
from ..settings import PERSISTATION_PIPELINE
from .authentication import AppAuthentication


class LogRecordResource(ModelResource):

    extra = fields.CharField(attribute='_extra', blank=True, default='{}')

    def save(self, bundle, skip_errors=False):
        self.is_valid(bundle)
        if bundle.errors and not skip_errors:
            raise ImmediateHttpResponse(response=self.error_response(bundle.request, bundle.errors))
        self.authorized_create_detail(self.get_object_list(bundle.request), bundle)
        for pipeline in PERSISTATION_PIPELINE:
            module_str, authenticator_str = pipeline.rsplit('.', 1)
            module = import_module(module_str)
            persistor = getattr(module, authenticator_str)
            persistor(bundle.data)
        return bundle

    def hydrate(self, bundle):
        bundle.data['app_id'] = self._meta.authentication.get_app(bundle.request).pk
        return bundle

    def build_schema(self):
        schema = super(LogRecordResource, self).build_schema()
        schema['CONSTANTS'] = {
            'LEVEL': LogRecord.LEVEL._choices
        }
        return schema

    class Meta:
        excludes = ['_extra']
        list_allowed_methods = ['post']
        detail_allowed_methods = []
        queryset = LogRecord.objects.all()
        authentication = AppAuthentication()
        authorization = Authorization()


import json
import copy

from django.utils.dateparse import parse_datetime

from django_gearman_commands import submit_job

from ..models import LogRecord
from ..settings import GERMAN_TASK_NAME


def persist_record_async(data):
    submit_job(GERMAN_TASK_NAME, json.dumps(data))
    return True


def persist_record(data):
    data = copy.copy(data)
    data['timestamp'] = parse_datetime(data['timestamp'])
    LogRecord(**data).save()
    return True



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'LogRecord.request_id'
        db.add_column(u'log4django_logrecord', 'request_id',
                      self.gf('django.db.models.fields.CharField')(db_index=True, max_length=36, null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'LogRecord.request_id'
        db.delete_column(u'log4django_logrecord', 'request_id')


    models = {
        u'log4django.app': {
            'Meta': {'ordering': "('name',)", 'object_name': 'App'},
            'description': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'default': "'b3bf8fef-97bc-460b-a4ed-8848bfb3807e'", 'unique': 'True', 'max_length': '36'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'log4django.logrecord': {
            'Meta': {'ordering': "('-id',)", 'object_name': 'LogRecord'},
            '_extra': ('django.db.models.fields.TextField', [], {'default': "'{}'"}),
            'app': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'records'", 'to': u"orm['log4django.App']"}),
            'exception_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'exception_traceback': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fileName': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'lineNumber': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'loggerName': ('django.db.models.fields.CharField', [], {'max_length': '225'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'request_id': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '36', 'null': 'True', 'blank': 'True'}),
            'thread': ('django.db.models.fields.CharField', [], {'max_length': '225', 'null': 'True', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {})
        }
    }

    complete_apps = ['log4django']



from django.http import HttpResponse
from django.utils.importlib import import_module
from django.conf import settings

from .settings import AUTHENTICATION_PIPELINE


class Log4DjangoAuthenticationMiddleware(object):

    def process_request(self, request):
        if not settings.DEBUG:
            for pipeline in AUTHENTICATION_PIPELINE:
                module_str, authenticator_str = pipeline.rsplit('.', 1)
                module = import_module(module_str)
                authenticator = getattr(module, authenticator_str)
                if not authenticator(request):
                    return HttpResponse('Unathorized', status=401)


import logging
from datetime import datetime

from django.utils.timezone import make_aware, get_default_timezone
from django.conf import settings as django_settings


import jsonpickle

from .settings import DEFAULT_APP_ID

LogRecord = None


class BaseFormatter(logging.Formatter):

    DEFAULT_PROPERTIES = logging.LogRecord('', '', '', '', '', '', '', '').__dict__.keys()

    def _get_extra(self, record):
        """Standard record decorated with extra contextual information."""
        extra = {}
        contextual_extra = set(record.__dict__).difference(set(self.DEFAULT_PROPERTIES))

        try:
            contextual_extra.remove('message')
            contextual_extra.remove('asctime')
        except KeyError:
            pass

        for key in contextual_extra:
            extra[key] = getattr(record, key)
        return extra

    def _get_data(self, record):
        timestamp = datetime.fromtimestamp(record.created)
        if django_settings.USE_TZ:
            timestamp = make_aware(timestamp, get_default_timezone())

        return dict(
            loggerName=record.name, level=record.levelno, timestamp=timestamp,
            message=record.getMessage(), fileName=record.pathname, lineNumber=record.lineno,
            thread=record.thread, app_id=getattr(record, 'app_id', DEFAULT_APP_ID),
            request_id=record.request_id
        )


class ModelFormatter(BaseFormatter):

    def format(self, record):
        # Lazy models imports.
        global LogRecord
        if LogRecord is None:
            from .models import LogRecord

        # Basic record data.
        log_record = LogRecord(**self._get_data(record))

        # Exception data if available.
        if record.exc_info is not None:
            log_record.exception_message = str(record.exc_info[1])
            log_record.exception_traceback = self.formatException(record.exc_info)
        # Extra data.
        log_record.extra = self._get_extra(record)
        return log_record


class GearmanFormatter(BaseFormatter):

    def format(self, record):
        # Basic record data.
        record_dict = self._get_data(record)
        
        # Exception data if available.
        if record.exc_info is not None:
            record_dict['exception_message'] = str(record.exc_info[1])
            record_dict['exception_traceback'] = self.formatException(record.exc_info)
        # Extra data.
        record_dict['extra'] = self._get_extra(record)
        record_dict['timestamp'] = record_dict['timestamp'].isoformat()

        return jsonpickle.encode(record_dict)


from django.conf import settings as django_settings


PAGE_TITLE = getattr(django_settings, 'LOG4DJANGO_PAGE_TITLE', 'log4django')
CONNECTION_NAME = getattr(django_settings, 'LOG4DJANGO_CONNECTION_NAME', 'default')
DEFAULT_APP_ID = getattr(django_settings, 'LOG4DJANGO_DEFAULT_APP_ID', None)
GERMAN_TASK_NAME = getattr(django_settings, 'LOG4DJANGO_GEARMAN_TASK_NAME', 'log4django_event')
PAGE_SIZE = getattr(django_settings, 'LOG4DJANGO_PAGE_SIZE', 100)
PAGINATOR_RANGE = getattr(django_settings, 'LOG4DJANGO_PAGINATOR_RANGE', 15)
EXTRA_DATA_INDENT = getattr(django_settings, 'LOG4DJANGO_EXTRA_DATA_INDENT', 4)
AUTHENTICATION_PIPELINE = getattr(django_settings, 'LOG4DJANGO_AUTHENTICATION_PIPELINE', (
    'log4django.pipeline.authentication.is_logged',
))
PERSISTATION_PIPELINE = getattr(django_settings, 'LOG4DJANGO_PERSISTATION_PIPELINE', (
    'log4django.pipeline.process_bundle_data.persist_record',
))
CSV_EXPORT_EXTRA_JSON_PATHS = getattr(django_settings, 'LOG4DJANGO_CSV_EXPORT_EXTRA_JSON_PATHS', tuple())
CSV_DOWNLOAD_FILE_NAME =  getattr(django_settings, 'LOG4DJANGO_CSV_DOWNLOAD_FILE_NAME', 'log4django.csv')


from django.conf.urls import patterns, include, url

from tastypie.api import Api

from .api.resources import LogRecordResource
from .views.logrecord import LogRecordList, LogRecordDetail


api_patterns = Api(api_name='v1')
api_patterns.register(LogRecordResource())


urlpatterns = patterns('log4django.views',
    url(r'^api/', include(api_patterns.urls)),
    url(r'^export/csv/$', 'exports.csv', name='csv_export'),
    url(r'^$', LogRecordList.as_view(), name='logrecord_list'),
    url(r'^logrecord/(?P<logrecord_id>\d+)$', LogRecordDetail.as_view(), name='logrecord_detail')
)


import json

from django.db import transaction
from django.conf import settings

from django_gearman_commands import GearmanWorkerBaseCommand

from ...settings import GERMAN_TASK_NAME
from ...pipeline.process_bundle_data import persist_record


class Command(GearmanWorkerBaseCommand):

    @property
    def task_name(self):
        return GERMAN_TASK_NAME

    @transaction.commit_on_success(using=settings.DATABASE_ALIAS_LOGGING)
    def do_job(self, job_data):
        try:
            payload = json.loads(job_data)
            return persist_record(payload)
        except Exception, ex:
            print ex
        return False


# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages

import log4django


setup(
    name='log4django',
    version=log4django.get_version(),
    description='log4django is full features logging platform for django applications.',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author=u'Vladimír Gorej',
    author_email='gorej@codescale.net',
    url='http://www.codescale.net/en/community#log4django',
    download_url='http://github.com/CodeScaleInc/log4django/tarball/master',
    license='BSD',
    keywords='logging, django, log, logs',
    packages=find_packages('.'),
    include_package_data=True,
    install_requires = [
        'Django', 'django-model-utils', 'django-tastypie',
        'jsonpath', 'jsonpickle', 'mimeparse', 'python-dateutil',
        'django-log-request-id', 'south'
    ],
    extras_require = {
        'gearman': ['gearman==dev', 'django-gearman-commands==dev'],
        'tests': ['mock']
    },
    platforms='any',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Browsers'
    ],
    dependency_links = [
        'git+git://github.com/Yelp/python-gearman.git#egg=gearman-dev',
        'git+git://github.com/CodeScaleInc/django-gearman-commands.git#egg=django_gearman_commands-dev',
    ]
)



from django.core.urlresolvers import reverse
from django.test import TestCase

from . import TEST_USERNAME, TEST_PASSWORD


class MainScreenTest(TestCase):

    def setUp(self):
        self.client.login(username=TEST_USERNAME, password=TEST_PASSWORD)

    def test_context(self):
        response = self.client.get(reverse('log4django:main_screen'))
        self.assertIn('records', response.context)
        self.assertIn('apps', response.context)
        self.assertIn('loggers', response.context)
        self.assertIn('levels', response.context)
        self.assertIn('filter_levels', response.context)


import codecs
import StringIO

from django.utils.importlib import import_module
csv = import_module('csv')

import jsonpath

from ..settings import CSV_EXPORT_EXTRA_JSON_PATHS


class UnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        # Redirect output to a queue
        self.queue = StringIO.StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        self.writer.writerow([unicode(s).encode("utf-8") if s is not None else u'' for s in row])
        # Fetch UTF-8 output from the queue ...
        data = self.queue.getvalue()
        data = data.decode("utf-8")
        # ... and reencode it into the target encoding
        data = self.encoder.encode(data)
        # write to the target stream
        self.stream.write(data)
        # empty queue
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)


class Export(object):
    def __init__(self, queryset):
        self.queryset = queryset
        self.buffer = StringIO.StringIO()
        self.writer = UnicodeWriter(self.buffer)

    def render(self):
        for logrecord in self.queryset:
            row = [logrecord.app, logrecord.loggerName, logrecord.get_level_display(), logrecord.timestamp,
                   logrecord.message, logrecord.fileName, logrecord.lineNumber, logrecord.thread,
                   logrecord.exception_message]
            for json_path in CSV_EXPORT_EXTRA_JSON_PATHS:
                jsonpath_result = jsonpath.jsonpath(logrecord.extra, json_path)
                row.append(jsonpath_result[0] if jsonpath_result is not False else u'')
            self.writer.writerow(row)
        csv_data = self.buffer.getvalue()
        self.buffer.close()
        return csv_data



from .settings import CONNECTION_NAME


class Log4DjangoRouter(object):
    def db_for_read(self, model, **hints):
        if model._meta.app_label == 'log4django':
            return CONNECTION_NAME
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label == 'log4django':
            return CONNECTION_NAME
        return None

    def allow_relation(self, obj1, obj2, **hints):
        if obj1._meta.app_label == 'log4django' or obj2._meta.app_label == 'log4django':
            return True
        return None

    def allow_syncdb(self, db, model):
        if db == CONNECTION_NAME and model._meta.app_label == 'log4django':
            return True
        return None


from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.db.models import Q
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView

from ...models import LogRecord, App
from ...settings import PAGE_SIZE
from ...decorators import authenticate
from .. import _filter_records


class LogRecordList(TemplateView):
    template_name = 'log4django/bootstrap/logrecord/list.html'
    http_method_names = ('get',)

    @method_decorator(authenticate())
    def get(self, request, *args, **kwargs):
        logrecord_qs = _filter_records(request)
        paginator = Paginator(logrecord_qs, PAGE_SIZE)
        page = request.GET.get('page', None)
        try:
            records = paginator.page(page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            records = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            records = paginator.page(paginator.num_pages)
        # Getting filtering values.
        apps = App.objects.all()
        loggers = set([r.loggerName for r in records])
        levels = LogRecord.LEVEL
        return self.render_to_response(dict(
            records=records, apps=apps, loggers=loggers, levels=levels,
            filter_levels=[int(l) for l in request.GET.getlist('level')]
        ))


class LogRecordDetail(TemplateView):
    template_name = 'log4django/bootstrap/logrecord/detail.html'
    http_method_names = ('get',)

    @method_decorator(authenticate())
    def get(self, request, logrecord_id=None):
        record = get_object_or_404(LogRecord, pk=logrecord_id)

        related = None
        if record.request_id:
            related = LogRecord.objects.filter(
                Q(request_id=record.request_id)
                & ~Q(pk=record.pk)
            )

        return self.render_to_response(dict(
            record=record, related=related
        ))
